﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SteamClient
{

    public partial class MainForm : Form
    {
        public static bool m_USERISLOGGEDIN = false;
        public static string m_MOUNTPATH = Application.StartupPath + "\\Steamapps\\";
        public static string m_USERNAME = "";
        public static string m_USERLOGIN = "";
        public static string m_USEREMAIL = "";
        public static string m_USERPASSWORD = "";

        public static string pszOptionalPublicEncryptionKeyFilename = "";
        public static string pszOptionalPrivateDecryptionKeyFilename = "";
        public static string[] m_Args = new string[0];

        public static int iHandle = 0;

        private Dictionary<uint, CSteamApp> SteamContent = new Dictionary<uint, CSteamApp>();
        
        public MainForm(string[] Args)
        {
            m_Args = Args;
            InitializeComponent();
        }

        private void TestCallback(Steam_API.ESteamNotificationCallbackEvent eEvent, uint nData)
        {
            MessageBox.Show(nData.ToString() + "\x0D\x0A" + eEvent.ToString());
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            this.Width = 250;
            this.Height = 400;

            LoginBox.Top = 0;
            LoginBox.Left = 0;
            LoginBox.Visible = true;

            ClientBox.Top = 0;
            ClientBox.Left = 0;
            ClientBox.Visible = false;

            MainForm_Resize(sender, e);
            Steam_API.TSteamError pError = new Steam_API.TSteamError();
            int iStartEngine = 0;
            int iRet = 0;
            iStartEngine = Steam_API.SteamStartEngine(ref pError);
            iRet = Steam_API._f("Steam006");
            if (iStartEngine == 1 && iRet != 0)
            {
                int iStartup = 0;
                iStartup = Steam_API.SteamStartup(15, ref pError);
                Steam_API.SteamInitializeUserIDTicketValidator(ref pszOptionalPublicEncryptionKeyFilename, ref pszOptionalPrivateDecryptionKeyFilename, (uint)600, (uint)300, (uint)2000, (uint)32, (uint)100);
                //Steam_API.SteamNotificationCallback_t NotificationCallback = new Steam_API.SteamNotificationCallback_t(TestCallback);
                //iRet = Steam_API.SteamSetNotificationCallback(NotificationCallback, ref pError);
            }
            else
            {
                ShowError(pError);
            }
        }

        private void MainForm_Resize(object sender, EventArgs e)
        {
            if (m_USERISLOGGEDIN == true) {
                ClientBox.Width = this.Width;
                ClientBox.Height = this.Height;
                ContentList.Width = this.Width - 20;
            } else {
                LoginBox.Top = (this.Height - LoginBox.Height) / 2;
                LoginBox.Left = (this.Width - LoginBox.Width) / 2;
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Steam_API.SteamShutdownUserIDTicketValidator();
            if (Steam_API.ShutdownSteam(m_USERISLOGGEDIN) != true) {
                e.Cancel = true;
            }
        }

        private static void ShowError(Steam_API.TSteamError pError)
        {
            if (pError.eSteamError != 0)
            {
                MessageBox.Show("Error (" + pError.eSteamError + "): " + pError.eSteamError.ToString(), "Steam Error");
            }
        }

        private void cmdLoginCancel_Click(object sender, EventArgs e)
        {
            this.Close();
            Application.Exit();
        }

        private void cmdLoginOK_Click(object sender, EventArgs e)
        {
            if (txtLogin.Text == "")
            {
                MessageBox.Show("Invalid User Name, try again!", "Login");
                txtLogin.Focus();
                SendKeys.Send("{Home}+{End}");
            }
            else
            {
                if (txtPassword.Text == "")
                {
                    MessageBox.Show("Invalid Password, try again!", "Login");
                    txtPassword.Focus();
                    SendKeys.Send("{Home}+{End}");
                }
                else
                {
                    Steam_API.TSteamError pErrorLogin = new Steam_API.TSteamError();

                    cmdLoginOK.Enabled = false;
                    txtLogin.Enabled = false;
                    txtPassword.Enabled = false;

                    pErrorLogin = Steam_API.ProcessLogin(txtLogin.Text, txtPassword.Text);

                    if (pErrorLogin.eSteamError == 0)
                    {
                        int iRet = 0;
                        Steam_API.TSteamError pError = new Steam_API.TSteamError();
                        m_USERISLOGGEDIN = true;
                        m_USERLOGIN = txtLogin.Text;
                        m_USEREMAIL = Steam_API.GetEmailAddress();
                        m_USERPASSWORD = txtPassword.Text;
                        iRet = Steam_API.GetApps(ref SteamContent, ref pError);
                        if (iRet == 1 && pError.eSteamError == 0)
                        {
                            LoginBox.Visible = false;
                            foreach (CSteamApp App in SteamContent.Values)
                            {
                                //if (Steam_API.GetAppUpdateStats(App.SteamApp.uId, 3) > 0)
                                //{
                                    ListViewItem item_app = new System.Windows.Forms.ListViewItem(App.SteamApp.szName, Steam_API.GetAppIcon(App.SteamApp.uId, GameIcons), this.ContentList.Groups[0]);
                                    item_app.Tag = App.SteamApp.uId.ToString();
                                    this.ContentList.Items.Add(item_app);
                                //}
                            }
                            ClientBox.Visible = true;
                            MainForm_Resize(sender, e);
                            this.Text = "STEAM - " + m_USERLOGIN;
                        }
                        else
                        {
                            ShowError(pError);
                        }
                    }
                    else
                    {
                        cmdLoginOK.Enabled = true;
                        txtLogin.Enabled = true;
                        txtPassword.Enabled = true;
                        ShowError(pErrorLogin);
                    }
                }
            }
        }

        private void MenuItemProperties_Click(object sender, EventArgs e)
        {
            PropertiesForm frmProperties = new PropertiesForm();
            frmProperties.Show();
            frmProperties.load(SteamContent[uint.Parse(ContentList.FocusedItem.Tag.ToString())]);
        }

        private void ContentList_DoubleClick(object sender, EventArgs e)
        {
            LaunchApp();
        }

        private void MenuItemLaunch_Click(object sender, EventArgs e)
        {
            LaunchApp();
        }

        private void LaunchApp()
        {
            CSteamApp App = SteamContent[uint.Parse(ContentList.FocusedItem.Tag.ToString())];
            if (App.SteamApp.uNumLaunchOptions > 1)
            {
                LaunchOptionsForm frmLaunchOptions = new LaunchOptionsForm();
                frmLaunchOptions.Show(this);
                frmLaunchOptions.load(App);
            }
            else
            {
                LaunchForm frmLaunch = new LaunchForm();
                frmLaunch.Show(this);
                frmLaunch.load(App, 0, "");
            }
        }
    }
}