﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace SteamClient
{
    public class Steam_API
    {
        public const uint STEAM_USING_FILESYSTEM                        = 0x00000001;
        public const uint STEAM_USING_LOGGING                           = 0x00000002;
        public const uint STEAM_USING_USERID                            = 0x00000004;
        public const uint STEAM_USING_ACCOUNT                           = 0x00000008;
        public const uint STEAM_USING_ALL                               = 0x0000000f;

        public const int STEAM_MAX_PATH                                 = 255;
        public const int STEAM_QUESTION_MAXLEN                          = 255;
        public const int STEAM_SALT_SIZE                                = 8;

        public const int STEAM_CARD_NUMBER_SIZE                         = 16;
        public const int STEAM_CARD_HOLDERNAME_SIZE                     = 80;
        public const int STEAM_CARD_EXPYEAR_SIZE                        = 4;
        public const int STEAM_CARD_EXPMONTH_SIZE                       = 2;
        public const int STEAM_CARD_CVV2_SIZE                           = 10;
        public const int STEAM_BILLING_ADDRESS1_SIZE                    = 80;
        public const int STEAM_BILLING_ADDRESS2_SIZE                    = 80;
        public const int STEAM_BILLING_CITY_SIZE                        = 80;
        public const int STEAM_BILLING_ZIP_SIZE                         = 10;
        public const int STEAM_BILLING_STATE_SIZE                       = 40;
        public const int STEAM_BILLING_COUNTRY_SIZE                     = 40;
        public const int STEAM_BILLING_PHONE_SIZE                       = 20;
        public const int STEAM_TYPE_OF_PROOF_OF_PURCHASE_SIZE           = 20;
        public const int STEAM_PROOF_OF_PURCHASE_TOKEN_SIZE             = 200;
        public const int STEAM_EXTERNAL_ACCOUNTNAME_SIZE                = 80;
        public const int STEAM_EXTERNAL_ACCOUNTPASSWORD_SIZE            = 80;

        public enum ESteamSeekMethod
        {
	        eSteamSeekMethodSet = 0,
	        eSteamSeekMethodCur = 1,
	        eSteamSeekMethodEnd = 2
        }

        public enum ESteamBufferMethod
        {
	        eSteamBufferMethodFBF = 0,
	        eSteamBufferMethodNBF = 1
        }

        public enum ESteamError
        {
	        eSteamErrorNone													= 0,
	        eSteamErrorUnknown												= 1,
	        eSteamErrorLibraryNotInitialized								= 2,
	        eSteamErrorLibraryAlreadyInitialized							= 3,
	        eSteamErrorConfig												= 4,
	        eSteamErrorContentServerConnect									= 5,
	        eSteamErrorBadHandle											= 6,
	        eSteamErrorHandlesExhausted										= 7,
	        eSteamErrorBadArg												= 8,
	        eSteamErrorNotFound												= 9,
	        eSteamErrorRead													= 10,
	        eSteamErrorEOF													= 11,
	        eSteamErrorSeek													= 12,
	        eSteamErrorCannotWriteNonUserConfigFile							= 13,
	        eSteamErrorCacheOpen											= 14,
	        eSteamErrorCacheRead											= 15,
	        eSteamErrorCacheCorrupted										= 16,
	        eSteamErrorCacheWrite											= 17,
	        eSteamErrorCacheSession											= 18,
	        eSteamErrorCacheInternal										= 19,
	        eSteamErrorCacheBadApp											= 20,
	        eSteamErrorCacheVersion											= 21,
	        eSteamErrorCacheBadFingerPrint									= 22,

	        eSteamErrorNotFinishedProcessing								= 23,
	        eSteamErrorNothingToDo											= 24,
	        eSteamErrorCorruptEncryptedUserIDTicket							= 25,
	        eSteamErrorSocketLibraryNotInitialized							= 26,
	        eSteamErrorFailedToConnectToUserIDTicketValidationServer		= 27,
	        eSteamErrorBadProtocolVersion									= 28,
	        eSteamErrorReplayedUserIDTicketFromClient						= 29,
	        eSteamErrorReceiveResultBufferTooSmall							= 30,
	        eSteamErrorSendFailed											= 31,
	        eSteamErrorReceiveFailed										= 32,
	        eSteamErrorReplayedReplyFromUserIDTicketValidationServer		= 33,
	        eSteamErrorBadSignatureFromUserIDTicketValidationServer			= 34,
	        eSteamErrorValidationStalledSoAborted							= 35,
	        eSteamErrorInvalidUserIDTicket									= 36,
	        eSteamErrorClientLoginRateTooHigh								= 37,
	        eSteamErrorClientWasNeverValidated								= 38,
	        eSteamErrorInternalSendBufferTooSmall							= 39,
	        eSteamErrorInternalReceiveBufferTooSmall						= 40,
	        eSteamErrorUserTicketExpired									= 41,
	        eSteamErrorCDKeyAlreadyInUseOnAnotherClient						= 42,

	        eSteamErrorNotLoggedIn											= 101,
	        eSteamErrorAlreadyExists										= 102,
	        eSteamErrorAlreadySubscribed									= 103,
	        eSteamErrorNotSubscribed										= 104,
	        eSteamErrorAccessDenied											= 105,
	        eSteamErrorFailedToCreateCacheFile								= 106,
	        eSteamErrorCallStalledSoAborted									= 107,
	        eSteamErrorEngineNotRunning										= 108,
	        eSteamErrorEngineConnectionLost									= 109,
	        eSteamErrorLoginFailed											= 110,
	        eSteamErrorAccountPending										= 111,
	        eSteamErrorCacheWasMissingRetry									= 112,
	        eSteamErrorLocalTimeIncorrect									= 113,
	        eSteamErrorNetwork												= 200
        }

        public enum EDetailedPlatformErrorType
        {
	        eNoDetailedErrorAvailable,
	        eStandardCerrno,
	        eWin32LastError,
	        eWinSockLastError,
	        eDetailedPlatformErrorCount
        }

        public enum	ESteamFindFilter					/* Filter elements returned by SteamFind{First,Next} */
        {
	        eSteamFindLocalOnly,			/* limit search to local filesystem */
	        eSteamFindRemoteOnly,			/* limit search to remote repository */
	        eSteamFindAll					/* do not limit search (duplicates allowed) */
        }

        /******************************************************************************
        **
        ** Exported structure and complex type definitions.
        **
        ******************************************************************************/

        public struct TSteamError
        {
	        public ESteamError eSteamError;
	        public EDetailedPlatformErrorType eDetailedErrorType;
	        public int nDetailedErrorCode;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 255)]
	        public string szDesc;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 4)]
        public struct TSteamElemInfo
        {
	        public int bIsDir;						/* If non-zero, element is a directory; if zero, element is a file */
	        public uint uSizeOrCount;		        /* If element is a file, this contains size of file in bytes */
	        public int bIsLocal;					/* If non-zero, reported item is a standalone element on local filesystem */
	        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 255)]
            public string cszName;	                /* Base element name (no path) */
	        public long lLastAccessTime;			/* Seconds since 1/1/1970 (like time_t) when element was last accessed */
	        public long lLastModificationTime;		/* Seconds since 1/1/1970 (like time_t) when element was last modified */
	        public long lCreationTime;				/* Seconds since 1/1/1970 (like time_t) when element was created */
        }

        public struct TSteamSubscriptionStats
        {
	        public uint uNumSubscriptions;
	        public uint uMaxNameChars;
	        public uint uMaxApps;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 4)]
        public struct TSteamAppDependencyInfo
        {
            public uint AppId;
            public uint IsRequired;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 255)]
	        public string szMountName;
        }

        public struct TSteamAppStats
        {
	        public uint uNumApps;
	        public uint uMaxNameChars;
	        public uint uMaxVersionLabelChars;
	        public uint uMaxLaunchOptions;
	        public uint uMaxLaunchOptionDescChars;
	        public uint uMaxLaunchOptionCmdLineChars;
	        public uint uMaxNumIcons;
	        public uint uMaxIconSize;
        }

        public struct TSteamAppVersion
        {
	        public string szLabel;
	        public uint uMaxLabelChars;
	        public uint uVersionId;
	        public int bIsNotAvailable;
        }

        public struct TSteamAppLaunchOption
        {
	        public string szDesc;
	        public uint uMaxDescChars;
	        public string szCmdLine;
	        public uint uMaxCmdLineChars;
	        public uint uIndex;
	        public uint uIconIndex;
	        public int bNoDesktopShortcut;
	        public int bNoStartMenuShortcut;
	        public int bIsLongRunningUnattended;
        }

        public struct TSteamApp
        {
            public string szName;
            public uint uMaxNameChars;
            public string szLatestVersionLabel;
	        public uint uMaxLatestVersionLabelChars;
	        public string szCurrentVersionLabel;
	        public uint uMaxCurrentVersionLabelChars;
	        public string szCacheFile;
            public uint uMaxCacheFileChars;
	        public uint uId;
	        public uint uLatestVersionId;
	        public uint uCurrentVersionId;
	        public uint uMinCacheFileSizeMB;
	        public uint uMaxCacheFileSizeMB;
	        public uint uNumLaunchOptions;
	        public uint uNumIcons;
	        public uint uNumVersions;
	        public uint numDependencies;
        }

        public struct TSteamSubscription
        {
            public string szName;
            public uint uMaxNameChars;
            public uint puAppIds;
            public uint uMaxAppIds;
            public uint uId;
            public uint uNumApps;
            public uint uDurationInSeconds;
            public uint uCostInCents;
            public uint bIsAutoRenewing;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 4)]
        public struct TSteamProgress
        {
            public int bValid;                        // non-zero if call provides progress info
            public uint uPercentDone;                 // 0 to 100 if bValid
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 255)]
            public string szProgress;               // additional progress info
        }

        public enum ESteamNotificationCallbackEvent
        {
            eSteamNotifyTicketsWillExpire,
            eSteamNotifyAccountInfoChanged,
            eSteamNotifyContentDescriptionChanged,
            eSteamNotifyPleaseShutdown,
            eSteamNotifyNewSponsorUrl,
            eSteamNotifyAppVersionChanged
        }

        //public struct SteamSalt_t
        //{
	    //    unsigned char uchSalt[STEAM_SALT_SIZE];
        //}

        public enum ESteamPaymentCardType
        {
            eVisa				= 1,
            eMaster				= 2,
            eAmericanExpress	= 3,
            eDiscover			= 4,
            eDinnersClub		= 5
        }

        public struct TSteamPaymentCardInfo
        {
	        public ESteamPaymentCardType eCardType;
	        public string szCardNumber;
	        public string szCardHolderName;
	        public string szCardExpYear;
	        public string szCardExpMonth;
	        public string szCardCVV2;
	        public string szBillingAddress1;
	        public string szBillingAddress2;
	        public string szBillingCity;
	        public string szBillingZip;
	        public string szBillingState;
	        public string szBillingCountry;
	        public string szBillingPhone;
        }

        //public struct TSteamPrepurchaseInfo
        //{
        //    public string szTypeOfProofOfPurchase[ STEAM_TYPE_OF_PROOF_OF_PURCHASE_SIZE + 1 ];
        //    // A ProofOfPurchase token is not necessarily a nul-terminated string; it may be binary data
        //    // (perhaps encrypted). Hence we need a length and an array of bytes.
        //    public uint	uLengthOfBinaryProofOfPurchaseToken;
        //    public string cBinaryProofOfPurchaseToken[ STEAM_PROOF_OF_PURCHASE_TOKEN_SIZE + 1 ];
        //}

        //public struct TSteamExternalBillingInfo
        //{
	    //    public string szAccountName[ STEAM_EXTERNAL_ACCOUNTNAME_SIZE + 1 ];
	    //    public string szPassword[ STEAM_EXTERNAL_ACCOUNTPASSWORD_SIZE + 1 ];
        //} 

        public enum ESteamSubscriptionBillingInfoType
        {
            ePaymentCardInfo		= 1,
	        ePrepurchasedInfo		= 2,
	        eAccountBillingInfo		= 3,
	        eExternalBillingInfo	= 4		// indirect billing via ISP etc (not supported yet)
        }

        public struct TSteamSubscriptionBillingInfo
        {
            public ESteamSubscriptionBillingInfoType eBillingInfoType;
            //union {
            //    TSteamPaymentCardInfo			PaymentCardInfo;
            //    TSteamPrepurchaseInfo			PrepurchaseInfo;
            //    TSteamExternalBillingInfo		ExternalBillingInfo;
            //    char							bUseAccountBillingInfo;
            //};
        }

        public struct TSteamUpdateStats
        {
            public uint uPhysicalBytesReceived;
        }

        /******************************************************************************
        **
        ** More exported constants
        **
        ******************************************************************************/

        public const int STEAM_INVALID_HANDLE = 0;
        public const int STEAM_INVALID_CALL_HANDLE = 0;
        public const int STEAM_INACTIVE_USERIDTICKET_VALIDATION_HANDLE = 0;
        public const uint STEAM_USE_LATEST_VERSION = 0xFFFFFFFF;

        // Applications need to be able to authenticate Steam users from ANY instance.
        // So a SteamIDTicket contains SteamGlobalUserID, which is a unique combination of 
        // instance and user id.

        // SteamLocalUserID is an unsigned 64-bit integer.
        // For platforms without 64-bit int support, we provide access via a union that splits it into 
        // high and low unsigned 32-bit ints.  Such platforms will only need to compare LocalUserIDs 
        // for equivalence anyway - not perform arithmetic with them.
        public struct TSteamSplitLocalUserID
        {
            public uint	Low32bits;
            public uint	High32bits;
        }

        //public struct TSteamGlobalUserID
        //{
        //	public int m_SteamInstanceID;
        //	union
        //	{
        //		SteamLocalUserID_t		As64bits;
        //		TSteamSplitLocalUserID	Split;
        //	}						m_SteamLocalUserID;
        //
        //}

        //"Could not connect to Steam network"

        public delegate void SteamNotificationCallback_t(ESteamNotificationCallbackEvent eEvent, uint nData);

        internal static int GetPercent(uint Val1, uint Val2)
        {
            double Percent = (double)Val1;
            Percent /= Val2;
            Percent *= 100;
            if (Percent.ToString().IndexOf(",") != -1)
            {
                return int.Parse(Percent.ToString().Remove(Percent.ToString().IndexOf(",")));
            }
            else
            {
                return (int)Val1;
            }
        }
        internal static bool ShutdownSteam(bool UserIsLoggedIn)
        {
            int iRet = 0;
            TSteamError pError = new TSteamError();
            if (UserIsLoggedIn == true)
            {
                iRet = SteamLogout(ref pError);
            }
            if (SteamNumAppsRunning(ref pError) > 1) {
                return false;
            } else {
                while (SteamCleanup(ref pError) != 0)
                {
                    Application.DoEvents();
                }
                iRet = SteamShutdownEngine(ref pError);
                return true;
            }
        }
        internal static TSteamError ProcessLogin(string UserName, string UserPassword)
        {
            int iHandle = 0;
            TSteamError pError = new TSteamError();
            TSteamProgress pProgress = new TSteamProgress();
            iHandle = SteamLogin(UserName, UserPassword, 1, ref pError);

            while (SteamProcessCall(iHandle, ref pProgress, ref pError) == 0)
            {
                if (pError.eSteamError == ESteamError.eSteamErrorBadHandle)
                {
                    break;
                }
                Application.DoEvents();
            }

            return pError;
        }
        internal static string GetEmailAddress()
        {
            int iRet = 0;
            uint emailchars = 0;
            TSteamError pError = new TSteamError();
            StringBuilder Email = new StringBuilder(' ', 128);
            iRet = SteamGetCurrentEmailAddress(Email, 128, ref emailchars, ref pError);
            if (iRet == 1)
            {
                return Email.ToString();
            }
            else
            {
                return "";
            }
        }

        //uStatsType - Range 1 to 4
        internal static uint GetAppUpdateStats(uint uAppId, uint uStatsType)
        {
            int iRet = 0;
            TSteamError pError = new TSteamError();
            TSteamProgress pProgress = new TSteamProgress();
            TSteamUpdateStats tUpdateStats = new TSteamUpdateStats();
            iRet = SteamGetAppUpdateStats(uAppId, uStatsType, ref tUpdateStats, ref pError);
            if (iRet > 0 && pError.eSteamError == 0)
            {
                SteamBlockingCall(iRet, 1, ref pError);
                Application.DoEvents();
                /*
                while (SteamProcessCall(iRet, ref pProgress, ref pError) == 0)
                {
                    if (pError.eSteamError == ESteamError.eSteamErrorBadHandle)
                    {
                        break;
                    }
                    Application.DoEvents();
                }
                */
                return tUpdateStats.uPhysicalBytesReceived;
            }
            else
            {
                return 0;
            }
        }
        internal static uint GetAppCacheSize(uint uAppId)
        {
            int iRet = 0;
            uint uSize = 0;
            TSteamError pError = new TSteamError();
            //TSteamProgress pProgress = new TSteamProgress();
            iRet = SteamGetAppCacheSize(uAppId, ref uSize, ref pError);
            if (iRet > 0 && pError.eSteamError == 0)
            {
                SteamBlockingCall(iRet, 1, ref pError);
                Application.DoEvents();
                /*
                while (SteamProcessCall(iRet, ref pProgress, ref pError) == 0)
                {
                    if (pError.eSteamError == ESteamError.eSteamErrorBadHandle)
                    {
                        break;
                    }
                    Application.DoEvents();
                }
                */
                return uSize;
            }
            else
            {
                return 0;
            }
        }

        internal static int GetAppIcon(uint uAppId, ImageList tImageList)
        {
            Icon tIcon;
            int iRet = 0;
            uint IconDataSize = 5120;
            TSteamError pError = new TSteamError();
            StringBuilder IconData = new StringBuilder(' ', (int)IconDataSize);
            iRet = SteamEnumerateAppIcon(uAppId, 0, IconData, IconDataSize, ref IconDataSize, ref pError);
            if (iRet == 1 && pError.eSteamError == ESteamError.eSteamErrorNone)
            {
                tIcon = new System.Drawing.Icon(IconData.ToString());
                MessageBox.Show(IconData.Length.ToString());
                tImageList.Images.Add(tIcon);
                return (tImageList.Images.Count - 1);
            }
            //else
            //{
            //    MessageBox.Show(pError.eSteamError.ToString());
            //}
            return -1;
        }
        internal static int GetApps(ref Dictionary<uint, CSteamApp> SteamContent, ref TSteamError pError)
        {
            int iRet = 0;
            TSteamAppStats AppStats = new TSteamAppStats();
            iRet = SteamGetAppStats(ref AppStats, ref pError);
            if (iRet == 1 && pError.eSteamError == ESteamError.eSteamErrorNone)
            {
                uint[] AppIds;
                AppIds = new uint[AppStats.uNumApps];
                iRet = SteamGetAppIds(AppIds, AppStats.uNumApps, ref pError);
                if (iRet == 1 && pError.eSteamError == ESteamError.eSteamErrorNone)
                {
                    foreach (uint AppId in AppIds)
                    {
                        if (AppId < 5 || AppId == 6 || AppId == 7 || AppId == 8) continue;
                        TSteamApp App = new TSteamApp();
                        
                        App.szName = new String(' ', 255);
                        App.uMaxNameChars = 255;
                        App.szCurrentVersionLabel = new String(' ', 255);
                        App.uMaxCurrentVersionLabelChars = 255;
                        App.szLatestVersionLabel = new String(' ', 255);
                        App.uMaxLatestVersionLabelChars = 255;
                        App.szCacheFile = new String(' ', 255);
                        App.uMaxCacheFileChars = 255;

                        iRet = SteamEnumerateApp(AppId, ref App, ref pError);
                        if (iRet == 1 && App.numDependencies > 0 && pError.eSteamError == ESteamError.eSteamErrorNone)
                        {
                            CSteamApp SApp = new CSteamApp();
                            SApp.SteamApp = App;
                            if ((SApp.State == "estateavailable" || SApp.State == "estatetool" || SApp.State == "") && SApp.IsDemo == 0)
                            {
                                SteamContent.Add(SApp.SteamApp.uId, SApp);
                            }
                        }
                        pError.eSteamError = 0;
                    }
                }
                else
                {
                    return 0;
                }
            }
            else
            {
                return 0;
            }
            return 1;
        }

        #region Interface
        
        [DllImport("steam.dll")]
        public static extern int CreateInterface(string cszSteamDLLAppsystemInterfaceVersion, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int _f(string cszSteamInterface);

        #endregion

        #region Initialization

        [DllImport("steam.dll")]
        public static extern int SteamStartEngine(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamStartup(uint uUsingMask, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamCleanup(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetVersion(StringBuilder szVersion, uint uVersionBufSize );

        [DllImport("steam.dll")]
        public static extern int SteamShutdownEngine(ref TSteamError pError);

        #endregion

        #region Asynchrounous call handling

        [DllImport("steam.dll")]
        public static extern int SteamProcessCall(int handle, ref TSteamProgress pProgress, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamAbortCall(int handle, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamBlockingCall(int handle, uint uiProcessTickMS, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamSetMaxStallCount(uint uNumStalls, ref TSteamError pError);

        #endregion

        #region Filesystem

        [DllImport("steam.dll")]
        public static extern int SteamMountAppFilesystem(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamUnmountAppFilesystem(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamMountFilesystem(uint uAppId, string szMountPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamUnmountFilesystem(string szMountPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamOpenFileEx(string cszFileName, string cszMode, uint puSize, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamOpenFile(string cszFileName, string cszMode, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamOpenTmpFile(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamFlushFile(uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamCloseFile(uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamSetvBuf(uint hFile, StringBuilder pBuf, ESteamBufferMethod eMethod, uint uBytes, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetc(uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamPutc(int cChar, uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamPrintFile(uint hFile, ref TSteamError pError, string cszFormat, string[] args);

        [DllImport("steam.dll")]
        public static extern uint SteamReadFile(StringBuilder pBuf, uint uSize, uint uCount, uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern uint SteamWriteFile(string pBuf, uint uSize, uint uCount, uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamSeekFile(uint hFile, long lOffset, ESteamSeekMethod esMethod, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern long SteamSizeFile(uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern long SteamTellFile(uint hFile, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamStat(string cszFileName, ref TSteamElemInfo pInfo, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamFindClose(uint hFind, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamFindFirst(string cszPattern, ref ESteamFindFilter eFilter, ref TSteamElemInfo pFindInfo, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamFindNext(uint hFind, ref TSteamElemInfo pFindInfo, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetLocalFileCopy(string cszFileName, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamIsFileImmediatelyAvailable(string cszName, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamHintResourceNeed(string cszHintList, int bForgetEverything, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamForgetAllHints(string cszMountPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamPauseCachePreloading(string cszMountPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamResumeCachePreloading(string cszMountPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamWaitForResources(string cszMasterList, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamFlushCache(uint uAppId, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetCacheDecryptionKey();

        [DllImport("steam.dll")]
        public static extern int SteamGetCacheDefaultDirectory(StringBuilder szPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamSetCacheDefaultDirectory(string szPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetCacheFilePath();

        [DllImport("steam.dll")]
        public static extern int SteamIsFileNeededByCache();

        [DllImport("steam.dll")]
        public static extern int SteamRepairOrDecryptCaches(uint uAppId, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamCreateCachePreloaders(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamIsCacheLoadingEnabled(uint uAppId, ref int pbIsLoading, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamLoadCacheFromDir(uint uAppId, string szPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamLoadFileToCache();

        [DllImport("steam.dll")]
        public static extern int SteamStartLoadingCache(uint uAppId, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamStopLoadingCache(uint uAppId, ref TSteamError pError);

        #endregion

        #region Logging

        [DllImport("steam.dll")]
        public static extern int SteamCreateLogContext(string cszName);

        [DllImport("steam.dll")]
        public static extern int SteamLog(uint hContext, string cszMsg);

        [DllImport("steam.dll")]
        public static extern void SteamLogResourceLoadStarted(string cszMsg);

        [DllImport("steam.dll")]
        public static extern void SteamLogResourceLoadFinished(string cszMsg);

        #endregion

        #region Account

        [DllImport("steam.dll")]
        public static extern int SteamCreateAccount(string cszUser, string cszPassphrase, string szCreationKey, string cszPersonalQuestion, string cszAnswerToQuestion, ref int pbCreated, uint uUnknown, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamDeleteAccount(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamChangeAccountName();

        [DllImport("steam.dll")]
        public static extern int SteamChangeEmailAddress(string cszNewEmailAddress, ref int pbChanged, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamChangeForgottenPassword();

        [DllImport("steam.dll")]
        public static extern int SteamChangePassword(string cszCurrentPassphrase, string cszNewPassphrase, ref int pbChanged, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamChangePersonalQA(string cszCurrentPassphrase, string cszNewPersonalQuestion, string cszNewAnswerToQuestion, ref int pbChanged, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateApp(uint AppId, ref TSteamApp pApp, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateAppDependency(uint AppId, uint uDependency, ref TSteamAppDependencyInfo pDependencyInfo, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateAppIcon(uint uAppId, uint uIconIndex, StringBuilder pIconData, uint uIconDataBufSize, ref uint puSizeOfIconData, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateAppLaunchOption(uint uAppId, uint uLaunchOptionIndex, ref TSteamAppLaunchOption pLaunchOption, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateAppVersion(uint uAppId, uint uVersionIndex, ref TSteamAppVersion pAppVersion, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateSubscription(uint uSubId, ref TSteamSubscription pSubscription, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateSubscriptionDiscount();

        [DllImport("steam.dll")]
        public static extern int SteamEnumerateSubscriptionDiscountQualifier();

        [DllImport("steam.dll")]
        public static extern int SteamGenerateSuggestedAccountNames();

        [DllImport("steam.dll")]
        public static extern int SteamGetAccountStatus();

        [DllImport("steam.dll")]
        public static extern int SteamGetAppCacheSize(uint uAppId, ref uint uCacheSize, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetAppDependencies();

        [DllImport("steam.dll")]
        public static extern int SteamGetAppDir();

        [DllImport("steam.dll")]
        public static extern int SteamGetAppIds(uint[] puAppIds, uint uMaxIds, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetAppPurchaseCountry(int appID, StringBuilder szCountryCode, uint a3, ref uint pPurchaseTime, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetAppStats(ref TSteamAppStats pAppStats, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetAppUpdateStats(uint uAppId, uint uStatType, ref TSteamUpdateStats pUpdateStats, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetAppUserDefinedInfo(uint uAppId, string cszPropertyName, StringBuilder szPropertyValue, uint uBufSize, ref uint puPropertyValueLength, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetAppUserDefinedRecord(uint uAppId, StringBuilder arg2, StringBuilder arg3, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetCurrentEmailAddress(StringBuilder szEmailaddress, uint uBufSize, ref uint puEmailaddressChars, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetNumAccountsWithEmailAddress();

        [DllImport("steam.dll")]
        public static extern int SteamGetSponsorUrl(uint uAppId, StringBuilder szUrl, uint uBufSize, ref uint pUrlChars, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetSubscriptionExtendedInfo();

        [DllImport("steam.dll")]
        public static extern int SteamGetSubscriptionIds(ref uint puSubIds, uint uMaxIds, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetSubscriptionPurchaseCountry();

        [DllImport("steam.dll")]
        public static extern int SteamGetSubscriptionReceipt();

        [DllImport("steam.dll")]
        public static extern int SteamGetSubscriptionStats(ref TSteamSubscriptionStats pSubscriptionStats, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetTotalUpdateStats(ref TSteamUpdateStats pUpdateStats, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetUser(StringBuilder szUser, uint uBufSize, ref uint puUserChars, int bIsSecureComputer, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamGetUserType();

        [DllImport("steam.dll")]
        public static extern int SteamIsAccountNameInUse();

        [DllImport("steam.dll")]
        public static extern int SteamIsAppSubscribed(uint uAppId, ref int pbIsAppSubscribed, ref int pReserved, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamIsLoggedIn(ref int pbIsLoggedIn, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamIsSecureComputer(ref int pbIsSecure, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamIsSubscribed(uint uSubscriptionId, ref int pbIsSubscribed, ref int pReserved, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamLaunchApp(uint uAppId, uint uLaunchOption, string cszArgs, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamLogin(string cszUser, string cszPassphrase, int bIsSecureComputer, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamLogout(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamMoveApp(uint uAppId, StringBuilder szPath, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamRefreshAccountInfo();

        [DllImport("steam.dll")]
        public static extern int SteamRefreshAccountInfoEx();

        [DllImport("steam.dll")]
        public static extern int SteamRefreshLogin(StringBuilder cszPassphrase, int bIsSecureComputer, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamRequestAccountsByCdKeyEmail();

        [DllImport("steam.dll")]
        public static extern int SteamRequestAccountsByEmailAddressEmail();

        [DllImport("steam.dll")]
        public static extern int SteamRequestEmailAddressVerificationEmail();

        [DllImport("steam.dll")]
        public static extern int SteamRequestForgottenPasswordEmail();

        [DllImport("steam.dll")]
        public static extern int SteamSetUser(string cszUser, ref int pbUserSet, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamSubscribe(uint uSubscriptionId, ref TSteamSubscriptionBillingInfo pSubscriptionBillingInfo, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamUnsubscribe(uint uSubscriptionId, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamUpdateAccountBillingInfo(ref TSteamPaymentCardInfo pPaymentCardInfo, int pbChanged, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamUpdateSubscriptionBillingInfo(uint uSubscriptionId, ref TSteamSubscriptionBillingInfo pSubscriptionBillingInfo, ref int pbChanged, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamVerifyEmailAddress();

        [DllImport("steam.dll")]
        public static extern int SteamVerifyPassword();

        [DllImport("steam.dll")]
        public static extern int SteamWaitForAppReadyToLaunch(uint uAppId, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamAckSubscriptionReceipt();

        [DllImport("steam.dll")]
        public static extern int SteamRemoveAppDependency();

        [DllImport("steam.dll")]
        public static extern int SteamSetAppCacheSize(uint uAppId, uint nCacheSizeInMb, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamSetAppVersion(uint uAppId, uint uAppVersionId, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamInsertAppDependency();

        [DllImport("steam.dll")]
        public static extern int SteamNumAppsRunning(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamFindApp();

        #endregion

        #region Minidump

        //[DllImport("steam.dll")]
        //public static extern int SteamWriteMiniDumpFromAssert();

        //[DllImport("steam.dll")]
        //public static extern int SteamWriteMiniDumpSetComment(const char *cszComment);

        //[DllImport("steam.dll")]
        //public static extern int SteamWriteMiniDumpUsingExceptionInfo();

        //[DllImport("steam.dll")]
        //public static extern int SteamWriteMiniDumpUsingExceptionInfoWithBuildId();
        
        #endregion

        #region User ID functions

        //[DllImport("steam.dll")]
        //public static extern ESteamError SteamGetEncryptedUserIDTicket(const void *pEncryptionKeyReceivedFromAppServer, uint uEncryptionKeyLength, void *pOutputBuffer, uint uSizeOfOutputBuffer, uint *pReceiveSizeOfEncryptedTicket, TSteamError *pError);
        
        [DllImport("steam.dll")]
        public static extern ESteamError SteamInitializeUserIDTicketValidator(ref string pszOptionalPublicEncryptionKeyFilename, ref string pszOptionalPrivateDecryptionKeyFilename, uint ClientClockSkewToleranceInSeconds, uint ServerClockSkewToleranceInSeconds, uint MaxNumLoginsWithinClientClockSkewTolerancePerClient, uint HintPeakSimultaneousValidations, uint AbortValidationAfterStallingForNProcessSteps);
        
        [DllImport("steam.dll")]
        public static extern ESteamError SteamShutdownUserIDTicketValidator();

        //[DllImport("steam.dll")]
        //public static extern string SteamGetEncryptionKeyToSendToNewClient(uint * pReceiveSizeOfEncryptionKey);
        
        //[DllImport("steam.dll")]
        //public static extern ESteamError SteamStartValidatingUserIDTicket(void *pEncryptedUserIDTicketFromClient, uint uSizeOfEncryptedUserIDTicketFromClient, uint ObservedClientIPAddr, SteamUserIDTicketValidationHandle_t *pReceiveHandle);

        //[DllImport("steam.dll")]
        //public static extern ESteamError SteamStartValidatingNewValveCDKey(void *pEncryptedNewValveCDKeyFromClient, uint uSizeOfEncryptedNewValveCDKeyFromClient, uint ObservedClientIPAddr, struct sockaddr *pPrimaryValidateNewCDKeyServerSockAddr, struct sockaddr *pSecondaryValidateNewCDKeyServerSockAddr, SteamUserIDTicketValidationHandle_t *pReceiveHandle);
        
        //[DllImport("steam.dll")]
        //public static extern ESteamError SteamProcessOngoingUserIDTicketValidation(int Handle, ref TSteamGlobalUserID pReceiveValidSteamGlobalUserID, uint *pReceiveClientLocalIPAddr, unsigned char *pOptionalReceiveProofOfAuthenticationToken, size_t SizeOfOptionalAreaToReceiveProofOfAuthenticationToken, size_t *pOptionalReceiveSizeOfProofOfAuthenticationToken);

        //[DllImport("steam.dll")]
        //public static extern void SteamAbortOngoingUserIDTicketValidation(int Handle);
        
        //[DllImport("steam.dll")]
        //public static extern ESteamError SteamOptionalCleanUpAfterClientHasDisconnected(uint ObservedClientIPAddr, uint ClientLocalIPAddr);
        
        #endregion

        #region Misc

        [DllImport("steam.dll")]
        public static extern void SteamClearError(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int InternalSteamNumClientsConnectedToEngine();

        [DllImport("steam.dll")]
        public static extern int InternalSteamShouldShutdownEngine2();

        [DllImport("steam.dll")]
        public static extern int SteamGetLocalClientVersion(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamChangeOfflineStatus();

        [DllImport("steam.dll")]
        public static extern int SteamGetOfflineStatus(ref uint buIsOffline, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamUninstall(ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamWeakVerifyNewValveCDKey();

        [DllImport("steam.dll")]
        public static extern int SteamGetEncryptedNewValveCDKey();

        [DllImport("steam.dll")]
        public static extern int SteamDecryptDataForThisMachine();

        [DllImport("steam.dll")]
        public static extern int SteamEncryptDataForThisMachine();

        [DllImport("steam.dll")]
        public static extern int SteamFindServersGetErrorString();

        [DllImport("steam.dll")]
        public static extern int SteamFindServersIterateServer(int arg1, int arg2, string szServerAddress, uint uServerAddressChars);

        [DllImport("steam.dll")]
        public static extern int SteamFindServersNumServers(uint arg1);

        [DllImport("steam.dll")]
        public static extern int SteamGetContentServerInfo(uint uAppId, ref TSteamError pError);

        [DllImport("steam.dll")]
        public static extern int SteamRefreshMinimumFootprintFiles();

        [DllImport("steam.dll")]
        public static extern int SteamSetNotificationCallback(SteamNotificationCallback_t pCallbackFunction, ref TSteamError pError);

        #endregion
    }
}
